searchData={"items":[{"type":"module","title":"ExSTUN","doc":"Module with helper functions.","ref":"ExSTUN.html"},{"type":"function","title":"ExSTUN.stun?/1","doc":"Checks if binary is a STUN message.","ref":"ExSTUN.html#stun?/1"},{"type":"module","title":"ExSTUN.Message","doc":"STUN Message\n\n```ascii\n      0                   1                   2                   3\n    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1\n   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+\n   |0 0|     STUN Message Type     |         Message Length        |\n   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+\n   |                         Magic Cookie                          |\n   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+\n   |                                                               |\n   |                     Transaction ID (96 bits)                  |\n   |                                                               |\n   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+\n\n                Figure 2: Format of STUN Message Header\n```","ref":"ExSTUN.Message.html"},{"type":"function","title":"ExSTUN.Message.add_attribute/2","doc":"Adds attribute to a message.","ref":"ExSTUN.Message.html#add_attribute/2"},{"type":"function","title":"ExSTUN.Message.authenticate/2","doc":"Authenticates a message.\n\n`key` depends on the authentication method.\nWhen authenticating using short-term mechanism, it is simply a password.\nWhen authenticating using long-term mechanism, use `lt_key/3` to obtain the key.\n\nPresence of username, realm and nonce attributes is not checked.\nDepending on the authentication method and its context (client/server side),\nuser has to perform those checks on their own.","ref":"ExSTUN.Message.html#authenticate/2"},{"type":"function","title":"ExSTUN.Message.check_fingerprint/1","doc":"","ref":"ExSTUN.Message.html#check_fingerprint/1"},{"type":"function","title":"ExSTUN.Message.decode/1","doc":"Decodes a binary into a STUN message.","ref":"ExSTUN.Message.html#decode/1"},{"type":"function","title":"ExSTUN.Message.encode/1","doc":"Encodes a STUN message into a binary.","ref":"ExSTUN.Message.html#encode/1"},{"type":"function","title":"ExSTUN.Message.get_all_attributes/2","doc":"Gets all attributes of given type from the message.\n\n`attr_mod` is a module implementing `ExSTUN.Message.Attribute` behaviour.\nReturns `nil` if there is no attribute of given type.","ref":"ExSTUN.Message.html#get_all_attributes/2"},{"type":"function","title":"ExSTUN.Message.get_attribute/2","doc":"Gets first attribute of given type from a message.\n\n`attr_mod` is a module implementing `ExSTUN.Message.Attribute` behaviour.\nReturns `nil` if there is no attribute of given type.","ref":"ExSTUN.Message.html#get_attribute/2"},{"type":"function","title":"ExSTUN.Message.lt_key/3","doc":"Create longe-term authentication key.","ref":"ExSTUN.Message.html#lt_key/3"},{"type":"function","title":"ExSTUN.Message.new/2","doc":"Creates a new STUN message with a random transaction id.","ref":"ExSTUN.Message.html#new/2"},{"type":"function","title":"ExSTUN.Message.new/3","doc":"Creates a new STUN message.","ref":"ExSTUN.Message.html#new/3"},{"type":"function","title":"ExSTUN.Message.with_fingerprint/1","doc":"","ref":"ExSTUN.Message.html#with_fingerprint/1"},{"type":"function","title":"ExSTUN.Message.with_integrity/2","doc":"","ref":"ExSTUN.Message.html#with_integrity/2"},{"type":"type","title":"ExSTUN.Message.decode_error_t/0","doc":"Possible `decode/1` error reasons.\n\n* `:not_enough_data` - provided binary is less than 20 bytes\n* `:malformed_header` - improper message header e.g. invalid cookie\n* `:unknown_method` - unknown message type method\n* `:data_after_finderprint` - fingerprint attribute is followed by other forbidden attributes\n* `:malformed_attr_padding` - one or more attributes are not followed by\nlong enough padding or padding is not 0.","ref":"ExSTUN.Message.html#t:decode_error_t/0"},{"type":"type","title":"ExSTUN.Message.t/0","doc":"","ref":"ExSTUN.Message.html#t:t/0"},{"type":"behaviour","title":"ExSTUN.Message.Attribute","doc":"Behaviour defining a STUN attribute.\n\nEach module implementing this behaviour is\nexpected to define its own struct that will be\npassed to `c:to_raw/2`.","ref":"ExSTUN.Message.Attribute.html"},{"type":"callback","title":"ExSTUN.Message.Attribute.from_raw/2","doc":"Deserializes a raw attribute.\n\nMessage is passed as some of attributes may\nrequire transaction id for deserialization.","ref":"ExSTUN.Message.Attribute.html#c:from_raw/2"},{"type":"callback","title":"ExSTUN.Message.Attribute.to_raw/2","doc":"Serializes an attribute.\n\nMessage is passed as some of attributes may\nrequire transaction id for serialization.","ref":"ExSTUN.Message.Attribute.html#c:to_raw/2"},{"type":"callback","title":"ExSTUN.Message.Attribute.type/0","doc":"Returns attribute integer type.","ref":"ExSTUN.Message.Attribute.html#c:type/0"},{"type":"module","title":"ExSTUN.Message.Class","doc":"STUN Message Class","ref":"ExSTUN.Message.Class.html"},{"type":"function","title":"ExSTUN.Message.Class.from_value/1","doc":"Converts class from an integer into an atom.","ref":"ExSTUN.Message.Class.html#from_value/1"},{"type":"function","title":"ExSTUN.Message.Class.to_value/1","doc":"Converts class from an atom into an integer.","ref":"ExSTUN.Message.Class.html#to_value/1"},{"type":"type","title":"ExSTUN.Message.Class.t/0","doc":"","ref":"ExSTUN.Message.Class.html#t:t/0"},{"type":"module","title":"ExSTUN.Message.Method","doc":"STUN Message Method","ref":"ExSTUN.Message.Method.html"},{"type":"function","title":"ExSTUN.Message.Method.from_value/1","doc":"Converts method from an integer into an atom.","ref":"ExSTUN.Message.Method.html#from_value/1"},{"type":"function","title":"ExSTUN.Message.Method.to_value/1","doc":"Converts method from an atom into an integer.","ref":"ExSTUN.Message.Method.html#to_value/1"},{"type":"type","title":"ExSTUN.Message.Method.t/0","doc":"","ref":"ExSTUN.Message.Method.html#t:t/0"},{"type":"module","title":"ExSTUN.Message.RawAttribute","doc":"STUN Message Attribute\n\n```ascii\n    0                   1                   2                   3\n0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1\n+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+\n|         Type                  |            Length             |\n+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+\n|                         Value (variable)                ....\n+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+\n\n              Figure 4: Format of STUN Attributes\n```","ref":"ExSTUN.Message.RawAttribute.html"},{"type":"function","title":"ExSTUN.Message.RawAttribute.encode/1","doc":"Encodes attribute to binary.\n\nThis function adds padding to align attributes on 32 bit boundary.","ref":"ExSTUN.Message.RawAttribute.html#encode/1"},{"type":"function","title":"Example - ExSTUN.Message.RawAttribute.encode/1","doc":"iex> attr = %ExSTUN.Message.RawAttribute{type: 0x8022, value: \"my_ex_stun_client\"}\n    iex> ExSTUN.Message.RawAttribute.encode(attr)\n    <<0x8022::16, 17::16, \"my_ex_stun_client\"::binary, 0, 0, 0>>","ref":"ExSTUN.Message.RawAttribute.html#encode/1-example"},{"type":"type","title":"ExSTUN.Message.RawAttribute.t/0","doc":"","ref":"ExSTUN.Message.RawAttribute.html#t:t/0"},{"type":"module","title":"ExSTUN.Message.Type","doc":"STUN Message Type\n\n```ascii\n        0                 1\n        2  3  4 5 6 7 8 9 0 1 2 3 4 5\n       +--+--+-+-+-+-+-+-+-+-+-+-+-+-+\n       |M |M |M|M|M|C|M|M|M|C|M|M|M|M|\n       |11|10|9|8|7|1|6|5|4|0|3|2|1|0|\n       +--+--+-+-+-+-+-+-+-+-+-+-+-+-+\n Figure 3: Format of STUN Message Type Field\n```","ref":"ExSTUN.Message.Type.html"},{"type":"function","title":"ExSTUN.Message.Type.from_value/1","doc":"Converts integer into a type.","ref":"ExSTUN.Message.Type.html#from_value/1"},{"type":"function","title":"ExSTUN.Message.Type.to_value/1","doc":"Converts type into an integer.","ref":"ExSTUN.Message.Type.html#to_value/1"},{"type":"type","title":"ExSTUN.Message.Type.t/0","doc":"","ref":"ExSTUN.Message.Type.html#t:t/0"},{"type":"module","title":"ExSTUN.URI","doc":"Module representing STUN/TURN URI.\n\nImplementation of RFC 7064 and RFC 7065.\n\nWe could try to use URI module from Elixir\nbut RFC 7064 and RFC 7065 state:\n\n  While these two ABNF productions are defined in [RFC3986]\n  as components of the generic hierarchical URI, this does\n  not imply that the \"stun\" and \"stuns\" URI schemes are\n  hierarchical URIs.  Developers MUST NOT use a generic\n  hierarchical URI parser to parse a \"stun\" or \"stuns\" URI.","ref":"ExSTUN.URI.html"},{"type":"function","title":"ExSTUN.URI.parse/1","doc":"Parses URI string into `t:t/0`.","ref":"ExSTUN.URI.html#parse/1"},{"type":"function","title":"ExSTUN.URI.parse!/1","doc":"The same as parse/1 but raises on error.","ref":"ExSTUN.URI.html#parse!/1"},{"type":"type","title":"ExSTUN.URI.scheme/0","doc":"","ref":"ExSTUN.URI.html#t:scheme/0"},{"type":"type","title":"ExSTUN.URI.t/0","doc":"Type describing URI struct.\n\n`transport` denotes protocol that should be used\nby a client to connect to the STUN/TURN server.\n`nil` means that the client should try to connect with every protocol it supports.\nIf scheme indicates secure connection, transport is always set to `:tcp`.\nThis is based on [RFC 5928, sec. 3](https://datatracker.ietf.org/doc/html/rfc5928#section-3).","ref":"ExSTUN.URI.html#t:t/0"},{"type":"type","title":"ExSTUN.URI.transport/0","doc":"","ref":"ExSTUN.URI.html#t:transport/0"},{"type":"module","title":"ExSTUN.Message.Attribute.ErrorCode","doc":"STUN Message Attribute Error-Code","ref":"ExSTUN.Message.Attribute.ErrorCode.html"},{"type":"type","title":"ExSTUN.Message.Attribute.ErrorCode.t/0","doc":"","ref":"ExSTUN.Message.Attribute.ErrorCode.html#t:t/0"},{"type":"module","title":"ExSTUN.Message.Attribute.Fingerprint","doc":"STUN Message Attribute Fingerprint","ref":"ExSTUN.Message.Attribute.Fingerprint.html"},{"type":"type","title":"ExSTUN.Message.Attribute.Fingerprint.t/0","doc":"","ref":"ExSTUN.Message.Attribute.Fingerprint.html#t:t/0"},{"type":"module","title":"ExSTUN.Message.Attribute.MessageIntegrity","doc":"STUN Message Attribute Message-Integrity","ref":"ExSTUN.Message.Attribute.MessageIntegrity.html"},{"type":"type","title":"ExSTUN.Message.Attribute.MessageIntegrity.t/0","doc":"","ref":"ExSTUN.Message.Attribute.MessageIntegrity.html#t:t/0"},{"type":"module","title":"ExSTUN.Message.Attribute.Nonce","doc":"STUN Message Attribute Nonce","ref":"ExSTUN.Message.Attribute.Nonce.html"},{"type":"type","title":"ExSTUN.Message.Attribute.Nonce.t/0","doc":"","ref":"ExSTUN.Message.Attribute.Nonce.html#t:t/0"},{"type":"module","title":"ExSTUN.Message.Attribute.Realm","doc":"STUN Message Attribute Realm","ref":"ExSTUN.Message.Attribute.Realm.html"},{"type":"type","title":"ExSTUN.Message.Attribute.Realm.t/0","doc":"","ref":"ExSTUN.Message.Attribute.Realm.html#t:t/0"},{"type":"module","title":"ExSTUN.Message.Attribute.Software","doc":"STUN Message Attribute Software","ref":"ExSTUN.Message.Attribute.Software.html"},{"type":"type","title":"ExSTUN.Message.Attribute.Software.t/0","doc":"","ref":"ExSTUN.Message.Attribute.Software.html#t:t/0"},{"type":"module","title":"ExSTUN.Message.Attribute.Username","doc":"STUN Message Attribute Username","ref":"ExSTUN.Message.Attribute.Username.html"},{"type":"type","title":"ExSTUN.Message.Attribute.Username.t/0","doc":"","ref":"ExSTUN.Message.Attribute.Username.html#t:t/0"},{"type":"module","title":"ExSTUN.Message.Attribute.XORMappedAddress","doc":"STUN Message Attribute XOR Mapped Address\n\n```ascii\n0                   1                   2                   3\n0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1\n+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+\n|0 0 0 0 0 0 0 0|    Family     |         X-Port                |\n+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+\n|                X-Address (Variable)                           |\n+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+\n\n        Figure 6: Format of XOR-MAPPED-ADDRESS Attribute\n```","ref":"ExSTUN.Message.Attribute.XORMappedAddress.html"},{"type":"type","title":"ExSTUN.Message.Attribute.XORMappedAddress.t/0","doc":"","ref":"ExSTUN.Message.Attribute.XORMappedAddress.html#t:t/0"},{"type":"extras","title":"ExSTUN","doc":"# ExSTUN\n\n[![Hex.pm](https://img.shields.io/hexpm/v/ex_stun.svg)](https://hex.pm/packages/ex_stun)\n[![API Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/ex_stun)\n[![CI](https://img.shields.io/github/actions/workflow/status/elixir-webrtc/ex_stun/ci.yml?logo=github&label=CI)](https://github.com/elixir-webrtc/ex_stun/actions/workflows/ci.yml)\n[![codecov](https://codecov.io/gh/elixir-webrtc/ex_stun/branch/master/graph/badge.svg?token=7FJ64MDD0J)](https://codecov.io/gh/elixir-webrtc/ex_stun)\n\nImplementation of the STUN protocol in Elixir - [RFC 8489](https://datatracker.ietf.org/doc/html/rfc8489)","ref":"readme.html"},{"type":"extras","title":"Installation - ExSTUN","doc":"```elixir\ndef deps do\n  [\n    {:ex_stun, \"~> 0.2.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - ExSTUN","doc":"```elixir\nalias ExSTUN.Message\nalias ExSTUN.Message.Type\nalias ExSTUN.Message.Attribute.XORMappedAddress\n\n{:ok, socket} = :gen_udp.open(0, [{:active, false}, :binary])\n\nreq = \n  %Type{class: :request, method: :binding}\n  |> Message.new() \n  |> Message.encode()\n\n:ok = :gen_udp.send(socket, 'stun.l.google.com', 19302, req)\n{:ok, {_, _, resp}} = :gen_udp.recv(socket, 0)\n\n{:ok, msg} = Message.decode(resp)\nMessage.get_attribute(msg, XORMappedAddress)\n```","ref":"readme.html#usage"},{"type":"extras","title":"Benchmarks - ExSTUN","doc":"```\nOperating System: Linux\nCPU Information: Intel(R) Core(TM) i5-9600K CPU @ 3.70GHz\nNumber of Available Cores: 6\nAvailable memory: 15.55 GB\nElixir 1.16.0\nErlang 26.2.1\nJIT enabled: true\n\nBenchmark suite executing with the following configuration:\nwarmup: 2 s\ntime: 5 s\nmemory time: 2 s\nreduction time: 2 s\nparallel: 1\ninputs: none specified\nEstimated total run time: 3 min 29 s\n\nBenchmarking binding_request.decode ...\nBenchmarking binding_request.encode ...\nBenchmarking binding_response.decode ...\nBenchmarking binding_response.encode ...\nBenchmarking error_code.from_raw ...\nBenchmarking error_code.to_raw ...\nBenchmarking message_full.authenticate (long-term) ...\nBenchmarking message_full.authenticate (short-term) ...\nBenchmarking message_full.check_fingerprint ...\nBenchmarking message_full.decode ...\nBenchmarking message_full.encode ...\nBenchmarking new_transaction_id ...\nBenchmarking raw_attr.encode ...\nBenchmarking software.from_raw ...\nBenchmarking software.to_raw ...\nBenchmarking type.from_value ...\nBenchmarking type.to_value ...\nBenchmarking xor_mapped_address.from_raw ...\nBenchmarking xor_mapped_address.to_raw ...\nCalculating statistics...\nFormatting results...\n\nName                                             ips        average  deviation         median         99th %\nerror_code.to_raw                            36.93 M       27.08 ns  ±7981.76%          26 ns          39 ns\nsoftware.to_raw                              17.55 M       56.99 ns ±58293.86%          32 ns          47 ns\ntype.to_value                                13.21 M       75.68 ns ±53851.08%          47 ns          60 ns\nsoftware.from_raw                            12.70 M       78.76 ns ±48756.65%          39 ns          53 ns\nraw_attr.encode                               6.76 M      147.84 ns ±35284.68%          84 ns          99 ns\nerror_code.from_raw                           6.68 M      149.72 ns ±31869.06%          68 ns         308 ns\nxor_mapped_address.from_raw                   6.53 M      153.04 ns ±24195.15%          90 ns         317 ns\ntype.from_value                               5.93 M      168.77 ns ±29536.32%          89 ns         169 ns\nxor_mapped_address.to_raw                     4.83 M      206.94 ns ±18672.29%         154 ns         204 ns\nbinding_request.decode                        3.88 M      257.52 ns ±18794.26%         164 ns         405 ns\nbinding_response.decode                       2.58 M      387.01 ns ±12289.08%         258 ns         598 ns\nmessage_full.check_fingerprint                1.96 M      509.65 ns  ±8520.92%         410 ns         631 ns\nbinding_response.encode                       1.75 M      572.69 ns  ±7262.63%         463 ns         800 ns\nmessage_full.decode                           1.18 M      848.38 ns  ±3970.03%         719 ns        1054 ns\nnew_transaction_id                            0.98 M     1024.22 ns  ±1116.74%        1010 ns        1049 ns\nbinding_request.encode                        0.76 M     1321.81 ns  ±1477.79%        1272 ns        1540 ns\nmessage_full.authenticate (short-term)        0.35 M     2845.65 ns   ±876.74%        2516 ns        3779 ns\nmessage_full.authenticate (long-term)         0.26 M     3873.42 ns   ±532.72%        3423 ns        5354 ns\nmessage_full.encode                           0.21 M     4870.66 ns   ±604.07%        4115 ns        8119 ns\n\nExtended statistics: \n\nName                                           minimum        maximum    sample size                     mode\nerror_code.to_raw                                23 ns     7959548 ns        13.57 M                    26 ns\nsoftware.to_raw                                  28 ns   113335333 ns        12.84 M                    32 ns\ntype.to_value                                    42 ns   103387309 ns        12.87 M                    46 ns\nsoftware.from_raw                                35 ns   115844653 ns        12.51 M                    39 ns\nraw_attr.encode                                  79 ns   112984566 ns        11.40 M                    84 ns\nerror_code.from_raw                              63 ns   112746828 ns        11.17 M                    68 ns\nxor_mapped_address.from_raw                      84 ns    83446950 ns        10.82 M                    90 ns\ntype.from_value                                  81 ns    99367969 ns        10.89 M                    89 ns\nxor_mapped_address.to_raw                       146 ns    80528411 ns         9.65 M                   153 ns\nbinding_request.decode                          153 ns    94202921 ns         9.29 M                   162 ns\nbinding_response.decode                         240 ns    68201852 ns         7.57 M                   250 ns\nmessage_full.check_fingerprint                  390 ns    59291887 ns         6.27 M                   409 ns\nbinding_response.encode                         448 ns    59683037 ns         5.73 M                   463 ns\nmessage_full.decode                             681 ns    44644869 ns         4.46 M                   704 ns\nnew_transaction_id                              975 ns    20779566 ns         3.65 M                  1013 ns\nbinding_request.encode                         1225 ns    23295785 ns         2.94 M                  1271 ns\nmessage_full.authenticate (short-term)         2401 ns    14778234 ns         1.58 M                  2500 ns\nmessage_full.authenticate (long-term)          3243 ns     9581577 ns         1.18 M                  3368 ns\nmessage_full.encode                            3880 ns    19112426 ns       948.71 K                  4085 ns\n\nMemory usage statistics:\n\nName                                      Memory usage\nerror_code.to_raw                                  0 B\nsoftware.to_raw                                   48 B - ∞ x memory usage +48 B\ntype.to_value                                      0 B - 1.00x memory usage +0 B\nsoftware.from_raw                                 64 B - ∞ x memory usage +64 B\nraw_attr.encode                                   32 B - ∞ x memory usage +32 B\nerror_code.from_raw                              144 B - ∞ x memory usage +144 B\nxor_mapped_address.from_raw                      264 B - ∞ x memory usage +264 B\ntype.from_value                                  384 B - ∞ x memory usage +384 B\nxor_mapped_address.to_raw                        232 B - ∞ x memory usage +232 B\nbinding_request.decode                           648 B - ∞ x memory usage +648 B\nbinding_response.decode                         1024 B - ∞ x memory usage +1024 B\nmessage_full.check_fingerprint                   232 B - ∞ x memory usage +232 B\nbinding_response.encode                          688 B - ∞ x memory usage +688 B\nmessage_full.decode                             2928 B - ∞ x memory usage +2928 B\nnew_transaction_id                                72 B - ∞ x memory usage +72 B\nbinding_request.encode                           360 B - ∞ x memory usage +360 B\nmessage_full.authenticate (short-term)           664 B - ∞ x memory usage +664 B\nmessage_full.authenticate (long-term)            808 B - ∞ x memory usage +808 B\nmessage_full.encode                             2344 B - ∞ x memory usage +2344 B\n\n**All measurements for memory usage were the same**\n\nReduction count statistics:\n\nName                                   Reduction count\nerror_code.to_raw                                    0\nsoftware.to_raw                                      2 - ∞ x reduction count +2\ntype.to_value                                        5 - ∞ x reduction count +5\nsoftware.from_raw                                    3 - ∞ x reduction count +3\nraw_attr.encode                                      1 - ∞ x reduction count +1\nerror_code.from_raw                                  3 - ∞ x reduction count +3\nxor_mapped_address.from_raw                         13 - ∞ x reduction count +13\ntype.from_value                                      6 - ∞ x reduction count +6\nxor_mapped_address.to_raw                           10 - ∞ x reduction count +10\nbinding_request.decode                              11 - ∞ x reduction count +11\nbinding_response.decode                             17 - ∞ x reduction count +17\nmessage_full.check_fingerprint                      43 - ∞ x reduction count +43\nbinding_response.encode                             40 - ∞ x reduction count +40\nmessage_full.decode                                 53 - ∞ x reduction count +53\nnew_transaction_id                                   3 - ∞ x reduction count +3\nbinding_request.encode                              22 - ∞ x reduction count +22\nmessage_full.authenticate (short-term)              49 - ∞ x reduction count +49\nmessage_full.authenticate (long-term)               84 - ∞ x reduction count +84\nmessage_full.encode                                 99 - ∞ x reduction count +99\n\n**All measurements for reduction count were the same**\n```","ref":"readme.html#benchmarks"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}